﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using HeuristicLab.Operators;
using HeuristicLab.Core;
using HeuristicLab.Data;
using HeuristicLab.Parameters;
using HeuristicLab.Common;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;
using HeuristicLab.PluginInfrastructure;
using HeuristicLab.Optimization;

namespace HeuristicLab.Algorithms.ParticleSwarmOptimization {
  public class ValueModifierSelector : SingleSuccessorOperator {
    public ILookupParameter<Item> ValueParameter {
      get { return (ILookupParameter<Item>)Parameters["Value"]; }
    }

    public ConstrainedValueParameter<IOperator> ModifierParameter {
      get { return (ConstrainedValueParameter<IOperator>)Parameters["Modifier"]; }
    }

    public ValueModifierSelector() : base() {
      Parameters.Add(new LookupParameter<Item>("Value", "The double value to modify."));
      Parameters.Add(new ConstrainedValueParameter<IOperator>("Modifier", "Modifies the value", new ItemSet<IOperator>()));
    }

    public override IDeepCloneable Clone(Cloner cloner) {
      return new ValueModifierSelector(this, cloner);
    }

    protected ValueModifierSelector(ValueModifierSelector original, Cloner cloner)
      : base(original, cloner) {
    }

    [StorableConstructor]
    protected ValueModifierSelector(bool deserializing) : base(deserializing) { }

    public override IOperation Apply() {
      OperationCollection next = new OperationCollection();
      next.Add(ExecutionContext.CreateOperation(this.ModifierParameter.Value));
      next.Add(base.Apply());
      return next;
    }
  }
}
